/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

/*
This file gives example of how to create a MIDI_parser with hooks into your own MIDI input 
processing functions.  In this case, the processing is a MIDI scope.
*/

#include "print_parser.h"
#include "ParserRegistration.h"

int print_parser::foo = 0;

static void note_off(  MIDI_msg *m )
{
	tw.pr("chan %X note %3d off vel %3d\r", m->chan(), m->d1, m->d2 );
}

static void note_on(  MIDI_msg *m )
{
	tw.pr("time: %8lX chan %X note %3d on vel %3d\r", m->time, m->chan(), m->d1, m->d2 );
}

static void poly_press(  MIDI_msg *m )
{
	tw.pr("chan %X note %3d press   %3d\r", m->chan(), m->d1, m->d2 );
}

static void program_chn(  MIDI_msg *m )
{
	tw.pr("chan %X prog %3d\r", m->chan(), m->d1 );
}

static void chan_press(  MIDI_msg *m )
{
	tw.pr("chan %X press %3d\r", m->chan(), m->d1);
}

static void pitch_bend(  MIDI_msg *m ) 
{
	tw.pr("chan %X bend %d\r", m->chan(), m->bend() );
}

// for CONTROL_CHANGE()
static void modulation(  MIDI_msg *m )
{tw.pr( "MODULATION AT %d.", m->d2);}  

static void master_volume(  MIDI_msg *m )
{tw.pr(" MASTER VOLUME AT %d.", m->d2);}

static void local_control_switch(  MIDI_msg *m )
{
   // what is the meaning of channel for this message?
	switch (m->d2)
	{
		case   0: tw.pr("LOCAL CONTROL OFF.\r"); break;
		case 127: tw.pr("LOCAL CONTROL ON.\r");  break;
		default:  tw.pr("LOCAL CONTROL ERROR: NEITHER 0 NOR 127.\r");
	}
}

static void all_notes_off(  MIDI_msg *m )
{
//	#pragma unused(m)
	tw.pr( " ALL NOTES OFF.\r" );
}

static void channels_equals_voices(  MIDI_msg *m )
{
//	#pragma unused(m)
	tw.pr(" MONO MODE: ON (NUMBER OF CHANNELS EQUALS NUMBER OF VOICES).\r" );
}

static void set_channels(  MIDI_msg *m )
{tw.pr(" MONO MODE: ON (NUMBER OF CHANNELS SET TO %d).\r", m->d2 );}

static void omni_poly_mono(  MIDI_msg *m )
{
	switch (m->d1)
	{
	case 124:                                          // Omni Mode Off (All Notes Off)
			tw.pr( " omni off.\r" ); break;
	case 125:                                          // Omni Mode On (All Notes Off)
			tw.pr( " omni on.\r" ); break;
	case 126:                                          // Mono Mode On (Poly Mode Off/All Notes Off)
		if (m->d2 == 0)
			channels_equals_voices(m);
		else
			set_channels(m);
		break;
	case 127:                                          // Poly Mode On (Mono Mode Off/All Notes Off)
		tw.pr(" POLY MODE: ON.\r" ); break;
	}
}

static void control_chn(  MIDI_msg *m)
{
	tw.pr( "chan %X ", m->chan() );
	switch (m->d1) {
	case 1:                                            // Continuous Controller 1 MSB (Modulation bender)
		modulation(m); break;
	case 7:                                            // Volume
		master_volume(m); break;
	case 122:                                     // CHANNEL MODE MESSAGES:
	   local_control_switch(m); break;
	case 123:                                          // All Notes Off
		if (m->d2 == 0)
			all_notes_off(m);
		break;
	case 124:                                          // Omni Mode Off (All Notes Off)
	case 125:                                          // Omni Mode On (All Notes Off)
	case 126:                                          // Mono Mode On (Poly Mode Off/All Notes Off)
	case 127:                                          // Poly Mode On (Mono Mode Off/All Notes Off)
		omni_poly_mono(m);
		break;
	default:
		tw.pr(" control change %d", m->d1);
	}
	tw.pr( "\r" );
}

static void sys_ex(  MIDI_msg *m )
{tw.pr("sysex ID %d\r", m->d1);}

static void system_msg(  MIDI_msg *m )
{
	switch (m->d1)
	{
	// System Exclusive message
	case smsgt::sys_ex: sys_ex(m); break;
	
	// System Common messages
	case smsgt::mtc: 						tw.pr("MTC.\r");    break;
	case smsgt::song_position:       tw.pr("SONG POSITION: %d.\r", m->dat14() ); break;
	case smsgt::song_select:         tw.pr("SONG SELECT: SONG #%d.\r", m->d1);	break;
	case smsgt::sys_com_undefined_1: tw.pr("SYSTEM COMMON UNDEFINED 1.\r");      break;
	case smsgt::sys_com_undefined_2: tw.pr("SYSTEM COMMON UNDEFINED 2.\r");      break;
	case smsgt::tune_request:        tw.pr("TUNE REQUEST.\r");                   break;
	case smsgt::end_sys_ex:          tw.pr("END OF SYSTEM EXCLUSIVE.\r");        break;
	
	// System Realtime messages
	case smsgt::timing_clock:        tw.pr("TIMING CLOCK.\r");                   break;
	case smsgt::sys_rea_undefined_1: tw.pr("SYSTEM REALTIME UNDEFINED 0.\r");    break;
	case smsgt::start:               tw.pr("START.\r");                          break;
	case smsgt::cont:                tw.pr("CONTINUE.\r");                       break;
	case smsgt::stop:                tw.pr("STOP.\r");                           break;
	case smsgt::sys_rea_undefined_2: tw.pr("SYSTEM REALTIME UNDEFINED 1.\r");    break;
	case smsgt::active_sensing:      tw.pr("ACTIVE SENSING.\r");                 break;
	case smsgt::system_reset:        tw.pr("SYSTEM RESET.\r");                   break;
	default:
		tw.pr( "ERROR (non-fatal, but bad): unrecognized system message" );
	}
}

static OMS_app *oa;

print_parser::
print_parser( OMS_app *in_oa )
{
	msg_hnd[MIDI_msg::note_off]    = note_off;
	msg_hnd[MIDI_msg::note_on]     = note_on;
	msg_hnd[MIDI_msg::poly_press]  = poly_press;
	msg_hnd[MIDI_msg::control_chn] = control_chn;
	msg_hnd[MIDI_msg::program_chn] = program_chn;
	msg_hnd[MIDI_msg::chan_press]  = chan_press;
	msg_hnd[MIDI_msg::pitch_bend]  = pitch_bend;
	msg_hnd[MIDI_msg::system]  = system_msg;
	
	
	oa = in_oa;
	oa->open_input("some_input");
}

static parser* newFunc()
{
	return (new print_parser(oap));
}

static ParserRegistration reg(newFunc, "print_parser");

// Authors: Ben Denckla, Herb Yang
